@@ -1,6 +1,6 @@
 package Apache::PageKit;
 
-# $Id: PageKit.pm,v 1.68 2001/05/21 14:49:42 tjmather Exp $
+# $Id: PageKit.pm,v 1.94 2001/06/19 16:29:33 tjmather Exp $
 
 # required for UNIVERSAL->can
 require 5.005;
@@ -12,11 +12,13 @@ use Apache::URI ();
 use Apache::Cookie ();
 use Apache::Request ();
 use Apache::Util ();
+use Compress::Zlib ();
 use File::Find ();
 use HTML::FillInForm ();
 use HTML::Parser ();
 use HTML::Template ();
 use Mail::Mailer ();
+use Text::Iconv ();
 use XML::Parser ();
 
 $| = 1;
@@ -29,17 +31,22 @@ use Apache::PageKit::Model ();
 use Apache::PageKit::Session ();
 use Apache::PageKit::Config ();
 
-use Apache::Constants qw(OK REDIRECT DECLINED);
+use Apache::Constants qw(OK DONE REDIRECT DECLINED);
 
 use vars qw($VERSION);
-$VERSION = '1.02';
+$VERSION = '1.04';
+
+%Apache::PageKit::DefaultMediaMap = (
+				     pdf => 'application/pdf',
+				     wml => 'text/vnd.wap.wml',
+				     xml => 'application/xml');
 
 # typically called when Apache is first loaded, from <Perl> section
 # in httpd.conf file
 sub startup ($$$) {
   my ($class, $pkit_root, $server) = @_;
 
-  die 'must specify $server in startup.  Usage: Apache::PageKit->startup($pkit_root, $server)' unless $server;
+  die 'must specify $server variable in startup.  Usage: Apache::PageKit->startup($pkit_root, $server)' unless $server;
 
   my $s = Apache->server;
 
@@ -64,30 +71,21 @@ sub startup ($$$) {
   }
 
   # User defined base model class
-  my $model_base_class = $config->get_global_attr('model_base_class');
+  my $model_base_class = $config->get_global_attr('model_base_class') || "MyPageKit::Common";
   eval "require $model_base_class";
-#  eval "import $model_base_class";
+  if($@){
+    die "Failed to load $model_base_class - looked in $pkit_root/Model";
+  }
 
-  my $default_lang = $config->get_global_attr('default_lang') || 'en';
-  my $html_clean_level = $config->get_server_attr('html_clean_level');
-  my $can_edit = $config->get_server_attr('can_edit');
-
-  my $view = Apache::PageKit::View->new(view_dir => "$pkit_root/View",
-					content_dir => "$pkit_root/Content",
-					cache_dir => $view_cache_dir,
-					default_lang => $default_lang,
-					html_clean_level => $html_clean_level,
-					can_edit => $can_edit);
-
-  # clean and pre-parse templates
-  $view->preparse_templates;
-
-  my $change_owner_sub = sub {
-    chown $uid, $gid, $_;
+  # delete all cache files, since some of them might be stale
+  # and might not be checked for freshness, if reload is off
+  # even if reload is on, PageKit might change, so it should be refreshed
+  my $unlink_sub = sub {
+    -f && unlink;
   };
-
-  # change ownership of cache files so that children can read them
-  File::Find::find($change_owner_sub,$view_cache_dir);
+  File::Find::find($unlink_sub,$view_cache_dir);
+  $model_base_class->pkit_startup($pkit_root, $server, $config)
+    if $model_base_class->can('pkit_startup');
 }
 
 # object oriented method call, see Eagle p.65
@@ -112,8 +110,10 @@ sub handler ($$){
 	$pk->component_code($component_id);
       }
       $model->pkit_post_common_code if $model->can('pkit_post_common_code');
+      $pk->set_session_cookie;
       $pk->prepare_and_print_view;
     }
+    $model->pkit_cleanup_code if $model->can('pkit_cleanup_code');
   };
   if($@){
     if($pk){
@@ -161,9 +161,31 @@ sub params_as_string {
 
   if($exclude_param && @$exclude_param){
     my %exclude_param = map {$_ => 1} @$exclude_param;
-    return join ('&', map { Apache::Util::escape_uri("$_") ."=" . Apache::Util::escape_uri($args->{$_} || "")} grep !{exists $exclude_param{$_}}, grep !/^(pkit_logout|pkit_view|pkit_check_cookie)$/, keys %$args);
+    return join ('&', map { Apache::Util::escape_uri("$_") ."=" . Apache::Util::escape_uri($args->{$_} || "")} grep !{exists $exclude_param{$_}}, grep !/^pkit_(logout|view|check_cookie|messages|error_messages|lang)$/, keys %$args);
   } else {
-    return join ('&', map { Apache::Util::escape_uri("$_") ."=" . Apache::Util::escape_uri($args->{$_} || "")} grep !/^(pkit_logout|pkit_view|pkit_check_cookie)$/, keys %$args);
+    return join ('&', map { Apache::Util::escape_uri("$_") ."=" . Apache::Util::escape_uri($args->{$_} || "")} grep !/^pkit_(logout|view|check_cookie|messages|error_messages|lang)$/, keys %$args);
+  }
+}
+
+sub update_session {
+  my ($pk) = @_;
+  # keep recent sessions recent, if user is logged in
+  # that is sessions time out if user hasn't viewed in a page
+  # in recent_login_timeout seconds
+
+  my $session = $pk->{session};
+  return unless defined($session);
+  unless(exists($session->{pkit_inactivity_timeout})){
+    my $recent_login_timeout = $pk->{config}->get_global_attr('recent_login_timeout');
+    my $last_activity = $session->{pkit_last_activity};
+    if(defined($recent_login_timeout) && defined($last_activity) &&
+       $last_activity + $recent_login_timeout < time()){
+      # user has been inactive for recent_login_timeout seconds, timeout
+      $session->{pkit_inactivity_timeout} = 1;
+    } else {
+      # update last_activity timestamp
+      $session->{pkit_last_activity} = time();
+    }
   }
 }
 
@@ -192,7 +214,9 @@ sub prepare_page {
     return DECLINED if $uri =~ m/$files_match/;
   }
 
-  if(my $uri_prefix = $config->get_global_attr('uri_prefix')){
+  my $uri_prefix = $config->get_global_attr('uri_prefix');
+
+  if($uri_prefix){
     $uri =~ s(^/$uri_prefix)(/);
   }
 
@@ -208,8 +232,8 @@ sub prepare_page {
   } else {
     $host = $apr->headers_in->{'Host'};
 
-    # this is not quite right and should be fixed
-    $uri_with_query = 'http://' . $host . $uri;
+    $uri_prefix =~ s!/$!!g;
+    $uri_with_query = (($ENV{HTTPS} eq 'on') ? 'https' : 'http') . '://' . $host . '/' . $uri_prefix . $uri;
   }
 #  my $pkit_selfurl;
 
@@ -227,7 +251,7 @@ sub prepare_page {
   my $output_param_object = $pk->{output_param_object};
   my $fillinform_object = $pk->{fillinform_object};
 
-  $pk->{output_param_object}->param(PKIT_HOSTNAME => $host);
+  $output_param_object->param(PKIT_HOSTNAME => $host);
 
 #  my $pkit_done = Apache::Util::escape_uri($apr->param('pkit_done') || $uri_with_query);
   my $pkit_done = $apr->param('pkit_done') || $uri_with_query;
@@ -251,38 +275,75 @@ sub prepare_page {
     $pk->{page_id} = $model->pkit_get_default_page;
   }
 
+  # check Accent-Encoding or if the user_agent field for Unix/Mac Netscape
+  # to see if should gzip output
+  my $gzip_output = $config->get_global_attr("gzip_output") || "";
+  if ($gzip_output =~ m!^(all|static)$!){
+    if(($apr->header_in("Accept-Encoding") || "") =~ /gzip/){
+      $pk->{use_gzip} = $gzip_output;
+    } else {
+      my $user_agent = $apr->header_in("User-Agent");
+      # this regular expression borrowed from Apache::AxKit::ConfigReader::DoZip
+      if ($user_agent =~ m{
+			   ^Mozilla/
+			   \d+
+			   \.
+			   \d+
+			   [\s\[\]\w\-]+
+			   (
+			    \(X11 |
+			    Macint.+PPC,\sNav
+			   )
+			  }x
+	 ) {
+	$pk->{use_gzip} = $gzip_output;
+      }
+    }
+  }
+
   # redirect "not found" pages
   unless ($pk->page_exists($pk->{page_id})){
     # first try to see if we can find a static file that we
     # can return
     my $filename = $pk->static_page_exists($pk->{page_id});
     unless($filename){
-      $pk->{page_id} = $config->uri_match($pk->{page_id})
-	|| $config->get_global_attr('not_found_page')
-	  || $config->get_global_attr('default_page');
+      $pk->{page_id} = ($config->uri_match($pk->{page_id})
+	|| $config->get_global_attr('not_found_page'))
+	  || $model->pkit_get_default_page;
       unless ($pk->page_exists($pk->{page_id})){
 	# if not_found_page is static, the return DECLINED...
 	$filename = $pk->static_page_exists($pk->{page_id});
       }
     }
     if ($filename){
-      # set path_info to '', otherwise Apache tacks it on at the end
       require MIME::Types;
       my ($media_type, $content_encoding) = MIME::Types::by_suffix($filename);
       $apr->content_type($media_type);
+      if($media_type eq "text/html" && $pk->{use_gzip}){
+        my $gzipped = $view->get_static_gzip($filename);
+        if ($gzipped){
+          $apr->content_encoding("gzip");
+          $apr->send_http_header;
+          $apr->print($gzipped);
+          return DONE;
+        }
+      }
+
+      # set path_info to '', otherwise Apache tacks it on at the end
       $apr->path_info('');
       $apr->filename($filename);
       return DECLINED;
     }
-    $pk->{page_id} = $config->uri_match($pk->{page_id})
-      || $config->get_global_attr('not_found_page');
-    unless($pk->{page_id}){
-      $pk->{page_id} = $model->pkit_get_default_page;
-    }
   }
 
   my $auth_user;
 
+  # session handling
+  if($model->can('pkit_session_setup')){
+    $pk->setup_session;
+  }
+  my $session = $pk->{session};
+
   if($apr->param('pkit_login')){
     if ($pk->login){
       # if login is sucessful, redirect to (re)set cookie
@@ -308,24 +369,17 @@ sub prepare_page {
     $auth_user = $pk->authenticate;
   }
 
-  # session handling
-  if($model->can('pkit_session_setup')){
-    $pk->setup_session;
-  }
-
   if($auth_user){
     my $pkit_check_cookie = $apr->param('pkit_check_cookie');
     if(defined($pkit_check_cookie) && $pkit_check_cookie eq 'on'){
       $model->pkit_message("You have successfully logged in.");
     }
+    $pk->update_session;
 
     my $require_login = $config->get_page_attr($pk->{page_id},'require_login');
     if(defined($require_login) && $require_login eq 'recent'){
-      my $recent_login_timeout = $config->get_global_attr('recent_login_timeout');
-      my $last_activity = $pk->{session}->{last_activity};
-      if(defined($recent_login_timeout) && defined($last_activity) &&
-	 $last_activity + $recent_login_timeout < time()){
-	# user is logged in, but has not been active recently
+      if(exists($session->{pkit_inactivity_timeout})){
+	# user is logged in, but has had inactivity period
 
 	# display verify password form
 	$pk->{page_id} = $config->get_global_attr('verify_page') ||
@@ -333,6 +387,8 @@ sub prepare_page {
 	# pkit_done parameter is used to return user to page that they originally requested
 	# after login is finished
 	$output_param_object->param("pkit_done",$uri_with_query) unless $apr->param("pkit_done");
+
+#	$apr->connection->user(undef);
       }
     }
   } else {
@@ -361,6 +417,25 @@ sub prepare_page {
     }
   }
 
+  # get language
+  # get Locale settings
+  my ($lang, $accept_lang);
+
+  if($lang = $apr->param('pkit_lang')){
+    $session->{'pkit_lang'} = $lang if $session;
+  } elsif ($session && ! exists $pk->{is_new_session}){
+    $lang = $session->{'pkit_lang'} if exists $session->{'pkit_lang'};
+  } else {
+    unless ($lang = substr($apr->header_in('Accept-Language'),0,2)){
+      $lang = $config->get_global_attr('default_lang') || 'en';
+    }
+  }
+
+  # TEMP only for anidea.com site, until fix problems with localization in content
+  $output_param_object->param("PKIT_LANG_$lang" => 1);
+
+  $pk->{lang} = $lang;
+
   $model->pkit_common_code if $model->can('pkit_common_code');
 
   # run the page code!
@@ -393,29 +468,10 @@ sub prepare_page {
 sub open_view {
   my ($pk) = @_;
 
-  my $apr = $pk->{apr};
-  my $view = $pk->{view};
-  my $session = $pk->{session};
-
-  my $page_id = $pk->{page_id};
-  my $pkit_view = $apr->param('pkit_view') || 'Default';
-
-  # get language
-  # get Locale settings
-  my $lang;
-
-  if($lang = $apr->param('pkit_lang')){
-    $session->{'pkit_lang'} = $lang if $session;
-  } elsif ($session){
-    $lang = $session->{'pkit_lang'} if exists $session->{'pkit_lang'};
-  }
-  $lang ||= substr($apr->header_in('Accept-Language'),0,2);
-
-  # TEMP only, until fix problems with localization in content
-  $pk->{output_param_object}->param("PKIT_LANG_$lang" => 1);
+  my $pkit_view = $pk->{apr}->param('pkit_view') || 'Default';
 
   # open template file
-  $view->open_view($page_id, $pkit_view, $lang);
+  $pk->{view}->open_view($pk->{page_id}, $pkit_view, $pk->{lang});
 }
 
 sub prepare_and_print_view {
@@ -431,6 +487,7 @@ sub prepare_and_print_view {
   # set view fillinform_objects and associated_objects, if approriate
   my $fill_in_form = $config->get_page_attr($page_id,'fill_in_form') || 'yes';
   if($fill_in_form ne 'no'){
+    # $apr comes first, so that fillinform overrides request parameters
     $view->{fillinform_objects} = [$apr, $pk->{fillinform_object}];
   } else {
     $view->{fillinform_objects} = [$pk->{fillinform_object}];
@@ -445,22 +502,124 @@ sub prepare_and_print_view {
   # set up page template and run component code
   my $output_ref = $view->fill_in_view;
 
+  # determine output media type
+  my $pkit_view = $apr->param('pkit_view') || 'Default';
+  my $output_media = $config->get_view_attr($pkit_view, 'output_media') ||
+    $config->get_page_attr($page_id, 'output_media');
+
+  $output_media ||= $Apache::PageKit::DefaultMediaMap{$pkit_view};
+  $output_media ||= 'text/html';
+
   # set expires to now so prevent caching
   #$apr->no_cache(1) if $apr->param('pkit_logout') || $config->get_page_attr($pk->{page_id},'template_cache') eq 'no';
   # see http://support.microsoft.com/support/kb/articles/Q234/0/67.ASP
   # and http://www.pacificnet.net/~johnr/meta.html
-  my $browser_cache =  $config->get_page_attr($pk->{page_id},'browser_cache') || 'yes';
+  my $browser_cache =  $config->get_page_attr($page_id,'browser_cache') || 'yes';
   $apr->header_out('Expires','-1') if $apr->param('pkit_logout') || $browser_cache eq 'no' || $apr->connection->user;
 
-  $apr->content_type('text/html');
-  $apr->send_http_header if $apr->is_initial_req;
-  return if $apr->header_only;
+  my $retcharset;
+  my $default_output_charset = $view->{default_output_charset};
+  my @charsets = ();
+  if($output_media eq 'text/html'){
+    # first get accepted charsets from incoming Accept-Charset HTTP header
+    if(my $accept_charset = $apr->headers_in->{'Accept-Charset'}){
+      my $pos = 0;
+      foreach my $charset (split(/,\s*/, $accept_charset)) {
+        my $score;
+
+        ($charset, $score) = split(/;\s*q=/, $charset, 2);
+        $score = 1 unless (defined($score) && ($score =~ /^(\+|\-)?\d+(\.\d+)?$/));
+        $charset =~ s/iso/ISO/;
+        $charset =~ s/utf/UTF/;
+        $charset =~ s/(us\-)?ascii/US-ASCII/;
+        push @charsets, [ $charset, $score, $pos++ ];
+      }
+      @charsets = sort {$b->[1] <=> $a->[1] || $a->[2] <=> $b->[2] } @charsets;
+      # set a content-type perhaps we over write this later if we know about the charset for te output pages
+      $apr->content_type("text/html");
+    }
+  } elsif ($output_media eq 'application/pdf'){
+    # write output_media to file, using process number of Apache child process
+    my $view_cache_dir = $view->{cache_dir};
+
+    my $fo_file = "$view_cache_dir/$$.fo";
+    my $pdf_file = "$view_cache_dir/$$.pdf";
+    open FO_TEMPLATE, ">$fo_file";
+    print FO_TEMPLATE $$output_ref;
+    close FO_TEMPLATE;
+
+    # call Apache XML FOP
+    my $fop_command = $config->get_server_attr('fop_command') ||
+      $config->get_global_attr('fop_command');
+    die "fop_command not specifed in configuration - must specify to use Apache XML FOP to generate PDFs" unless $fop_command;
+    my $error_message = `$fop_command $fo_file $pdf_file 2>&1 1>/dev/null`;
+
+    if(-s "$pdf_file"){
+      $$output_ref = "";
+
+      open PDF_OUTPUT, "$pdf_file";
+      while(<PDF_OUTPUT>){
+	$$output_ref .= $_;
+      }
+      close PDF_OUTPUT;
+    } else {
+      die "Error processing template with Apache XML FOO: $error_message";
+    }
+    $apr->content_type($output_media);
+  } else {
+    # just set content_type
+    $apr->content_type($output_media);
+  }
+
+  if($pk->{use_gzip} eq 'all'){
+    $apr->content_encoding("gzip");
+  }
+
+  # for a head request
+  if ($apr->header_only) {
+    $apr->send_http_header if $apr->is_initial_req;
+    return;
+  }
 
   # call output filter, if applicable
   $model->pkit_output_filter($output_ref)
     if $model->can('pkit_output_filter');
 
-  $apr->print($$output_ref);
+  my $converted_data;
+  if ($output_media eq 'text/html'){
+    my $data;
+    while (@charsets){
+      $retcharset = (shift @charsets)->[0];
+      last if ($retcharset eq $default_output_charset);
+      eval {
+        my $converter = Text::Iconv->new($default_output_charset, $retcharset);
+        $converted_data = $converter->convert($$output_ref);
+      };
+      last if ($converted_data);
+      $retcharset = undef;
+    }
+
+    # correct the header
+    $apr->content_encoding('') unless ($retcharset);
+    $apr->content_type("text/html; charset=$retcharset") if ($retcharset);
+    $apr->content_type("text/html")                      unless ($retcharset);
+  }
+
+  $apr->send_http_header if $apr->is_initial_req;
+
+  if($converted_data){
+    if($pk->{use_gzip} eq 'all'){
+      $apr->print(Compress::Zlib::memGzip($converted_data));
+    } else {
+      $apr->print($converted_data);
+    }
+  } else {
+    if($pk->{use_gzip} eq 'all' && $retcharset){
+      $apr->print(Compress::Zlib::memGzip($$output_ref));
+    } else {
+      $apr->print($$output_ref);
+    }
+  }
 }
 
 sub new {
@@ -473,19 +632,32 @@ sub new {
 
   # set up contained objects
   my $pkit_root = $r->dir_config('PKIT_ROOT');
+  die "Must specify PerlSetVar PKIT_ROOT in httpd.conf file" unless $pkit_root;
   my $config_dir = $pkit_root . '/Config';
   my $content_dir = $pkit_root . '/Content';
   my $view_dir = $pkit_root . '/View';
   my $server = $r->dir_config('PKIT_SERVER');
+  die "Must specify PerlSetVar PKIT_SERVER in httpd.conf file" unless $server;
   my $config = $self->{config} = Apache::PageKit::Config->new(config_dir => $config_dir,
 						 server => $server);
   my $apr = $self->{apr} = Apache::Request->new($r, POST_MAX => $self->{config}->get_global_attr('post_max'));
-  my $model_base_class = $self->{config}->get_global_attr('model_base_class');
-  my $model = $self->{model} = $model_base_class->new(pkit_pk => $self);
+  my $model_base_class = $self->{config}->get_global_attr('model_base_class') || "MyPageKit::Common";
+
+  my $model;
+  eval {$model = $self->{model} = $model_base_class->new(pkit_pk => $self)};
+  if($@){
+    unless($model_base_class){
+      die "model_base_class not specified";
+    } else {
+      die "Model class $model_base_class has no new method";
+    }
+  }
 
   $self->{dbh} = $model->pkit_dbi_connect if $model->can('pkit_dbi_connect');
 
   my $default_lang = $config->get_global_attr('default_lang') || 'en';
+  my $default_input_charset = $config->get_global_attr('default_input_charset') || 'ISO-8859-1';
+  my $default_output_charset = $config->get_global_attr('default_output_charset') || 'ISO-8859-1';
   my $html_clean_level = $config->get_server_attr('html_clean_level');
   my $can_edit = $config->get_server_attr('can_edit');
   my $reload = $config->get_server_attr('reload');
@@ -498,6 +670,8 @@ sub new {
 					     content_dir => "$pkit_root/Content",
 					     cache_dir => $view_cache_dir,
 					     default_lang => $default_lang,
+					     default_input_charset => $default_input_charset,
+					     default_output_charset => $default_output_charset,
 					     reload => $reload,
 					     html_clean_level => $html_clean_level,
 					     input_param_object => $apr,
@@ -534,7 +708,7 @@ sub page_sub {
 
   return undef unless (defined &{$perl_sub});
 
-  my $model_base_class = $pk->{config}->get_global_attr('model_base_class');
+  my $model_base_class = $pk->{config}->get_global_attr('model_base_class') || "MyPageKit::Common";
 
   warn qq{For full preformance please add "use $class_package" in your $model_base_class or startup.pl script\n};
 
@@ -612,10 +786,11 @@ sub login {
 
   $ses_key || return 0;
 
-  # check if user has a saved session_id
-
   # allow user to view pages with require_login eq 'recent'
-  $session->{last_activity} = time();
+  if(exists $pk->{session}){
+    delete $pk->{session}->{pkit_inactivity_timeout};
+    $pk->{session}->{pkit_last_activity} = time();
+  }
 
   # save session
   delete $pk->{session};
@@ -707,14 +882,13 @@ sub logout {
 
 # get session_id from cookie
 sub setup_session {
-  my ($pk) = @_;
+  my ($pk, $auth_user) = @_;
 
   my $model = $pk->{model};
 
   my $ss = $model->pkit_session_setup;
-
   unless($ss->{session_store_class} && $ss->{session_lock_class}){
-    warn "failed to set up session";
+    warn "failed to set up session - session_store_class and session_lock_class must be defined";
     $pk->{session} = {};
     return;
   }
@@ -731,61 +905,53 @@ sub setup_session {
     $session_id = $scookie->value;
   }
 
-  my $is_new_session;
-
-  $is_new_session = 1 unless $session_id;
+  # this sets a flag so we know if we should send a cookie later...
+  $pk->{is_new_session} = 1 unless $session_id;
 
   # set up session handler class
   my %session;
 
   my $session_lock_class = $ss->{session_lock_class};
   my $session_store_class = $ss->{session_store_class};
-
+  tie %session, 'Apache::PageKit::Session', $session_id,
   {
-#    local $Apache::PageKit::Error::in_use = 'no';
-    tie %session, 'Apache::PageKit::Session', $session_id,
-    {
-     Lock => $session_lock_class,
-     Store => $session_store_class,
-     Generate => 'MD5',
-     Serialize => 'Storable',
-     create_unknown => 1,
-     %{$ss->{session_args}}
-    };
-  }
+   Lock => $session_lock_class,
+   Store => $session_store_class,
+   Generate => 'MD5',
+   Serialize => 'Storable',
+   create_unknown => 1,
+   lazy => 1,
+   %{$ss->{session_args}}
+  };
 
   $pk->{session} = \%session;
+}
 
-  if($is_new_session){
-    # set cookie in users browser
-    my $session_id = $session{'_session_id'};
+sub set_session_cookie {
+  my ($pk) = @_;
+
+  return unless exists $pk->{is_new_session};
+
+  my $session = $pk->{session};
+  my $apr = $pk->{apr};
+
+  if(my $session_id = tied(%$session)->getid){
+    # something was stored in session
     my $expires = $pk->{config}->get_global_attr('session_expires');
     my @cookie_domains = split(' ',$pk->{config}->get_server_attr('cookie_domain'));
     @cookie_domains = (undef) if @cookie_domains == 0;
     for my $cookie_domain (@cookie_domains){
       my $cookie = Apache::Cookie->new($apr,
-				     -name => 'pkit_session_id',
-				     -value => $session_id,
-				     -path => "/");
+				       -name => 'pkit_session_id',
+				       -value => $session_id,
+				       -path => "/");
       $cookie->domain($cookie_domain) if $cookie_domain;
       $cookie->expires($expires) if $expires;
       $cookie->bake;
     }
-  } else {
-    # keep recent sessions recent
-    # that is sessions time out if user hasn't viewed in a page 
-    # in recent_login_timeout seconds
-    my $now = time();
-
-    $session{last_activity} = $now;
-#      if $session{last_activity} && 
-#	$session{last_activity} + $config->get_global_attr('recent_login_timeout') >= $now;
+    # save for logging purposes (warning, undocumented and might go away)
+    $apr->notes(pkit_session_id => $session_id);
   }
-
-  # save for logging purposes (warning, undocumented and might go away)
-  $apr->notes(pkit_session_id => $session_id);
-
-  return $session_id;
 }
 
 # check to see if page has either template or perl code associated with it
@@ -925,29 +1091,16 @@ the Config and Content XML files and pre-parses the View template files.
 
 =head1 FREQUENTLY ASKED QUESTIONS
 
-Please look in here before you send me an email.
-
-1) I get a segmentation fault when I start the PageKit
-enabled Apache server.
-
-PageKit requires XML::Parser, which is incompatible with the expat
-library included in Apache.  You'll have to configure Apache with 
-C<--disable-rule=expat>.
-
-For more information see http://axkit.org/faq.xml under
-"I install AxKit and Apache segfaults when it starts".
-
-The mod_perl guide (http://perl.apache.org/guide) has an excellect
-section on compiling mod_perl and Apache from source.
+See http://www.pagekit.org/faq.html
 
 =head1 SEE ALSO
 
 L<Apache::Request>, L<HTML::FillInForm>, L<HTML::Template>,
-L<HTML::FormValidator>
+L<Data::FormValidator>
 
 =head1 VERSION
 
-This document describes Apache::PageKit module version 1.02
+This document describes Apache::PageKit module version 1.04
 
 =head1 NOTES
 
@@ -983,8 +1136,6 @@ Support Template-Toolkit templates as well as HTML::Template templates.
 
 Support for exposing Model objects using XML-RPC and/or SOAP.
 
-Gzipped output.
-
 Support for multiple transformations with stylesheets, and for filters.
 
 Associate sessions with authenticated user ID.
@@ -1000,19 +1151,25 @@ T.J. Mather (tjmather@anidea.com)
 Fixes, Bug Reports, Docs have been generously provided by:
 
   Boris Zentner
+  Ben Ausden
   Stu Pae
   Yann Kerherv
-  Ben Ausden
   Chris Burbridge
   Leonardo de Carvalho
+  Rob Falcon
+  Sheffield Nolan
+  David Raimbault
 
-Thanks!
+Also, thanks to Dan Von Kokorn for helping shape the initial architecture
+and for the invaluable support and advice. 
 
 =head1 COPYRIGHT
 
 Copyright (c) 2000, 2001 AnIdea Corporation.  All rights Reserved.  PageKit is a trademark
 of AnIdea Corporation.
 
+Parts of code Copyright (c) 2000, 2001 AxKit.com Ltd.
+
 =head1 LICENSE
 
 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
